'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ComplexHandler_1 = require("../daemon/ComplexHandler");
const LoggerBroker_1 = require("../universal/LoggerBroker");
const ProcessBootstrap_1 = require("../application/ProcessBootstrap");
const GlobalConfigProcessor_1 = require("../universal/GlobalConfigProcessor");
const globalConfigProcessor = GlobalConfigProcessor_1.GlobalConfigProcessor.getInstance();
/**
 * Class DebugApplicationLoader
 * For a profile.js application to debugging
 */
class DebugApplicationLoader {
    constructor(options) {
        this.options = options;
    }
    /**
     * Start debug application
     * @return {Promise<void>}
     */
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            globalConfigProcessor.getAllProperties();
            globalConfigProcessor.mergeProperties({
                logger: {
                    appLogger: {
                        stdoutLevel: 'ALL',
                        level: 'NONE'
                    }
                }
            });
            process.env.PANDORA_DEV = 'true';
            process.env.NODE_ENV = process.env.NODE_ENV || 'local';
            const mode = this.options.mode || 'procfile.js';
            const appName = this.options.appName || 'debug';
            const options = Object.assign({}, this.options, { mode, appName });
            if (!this.master) {
                if ('fork' === mode) {
                    this.master = new ProcessBootstrap_1.ProcessBootstrap(options.entryFile, options);
                }
                else {
                    this.master = new ComplexHandler_1.ComplexHandler(options);
                }
            }
            this.master.start().then(() => {
                console.log('Debug application start successful.');
            }).catch((err) => {
                LoggerBroker_1.consoleLogger.error(err);
            });
        });
    }
}
exports.DebugApplicationLoader = DebugApplicationLoader;
//# sourceMappingURL=DebugApplicationLoader.js.map